<?php
/*
Plugin Name: Good Comments URL Next
Plugin URI: http://a-panov.ru/good-comments-url-next/
Description: Удаляет url комментатора, если указанный сайт имеет низкое качество ресурса — параметр Яндекс ИКС.
Version: 0.2beta
Author: Andi Panov
Author URI: http://a-panov.ru
License: GPLv2
*/

/*
Для работы требуются:
— PHP 5.4 или выше;
— PHP с установленным расширением mbstring;
*/

// плагин работает только если определена соответствующая константа
if (!defined('WP_PLUGIN_DIR')) return false;

// путь к подключаемым файлам плагина
define('GCU_FILES', __DIR__ . DIRECTORY_SEPARATOR . 'inc' . DIRECTORY_SEPARATOR);

// если пользователь находится в админ-панели…
if (defined('WP_ADMIN')) {
    // … активировать хуки управления
    
    // добавим пункт в основное админ-меню WP
    add_action('admin_menu', ['GoodCommentsUrl', 'adminMenu']);
    
    // добавим ссылку на Настройки в списке плагинов
    add_filter('plugin_action_links', ['GoodCommentsUrl', 'gcuLinks'], 10, 2);
    
    // срабатывает сразу после удаления комментария
    add_filter('deleted_comment', ['GoodCommentsUrl', 'deleteСomment']);
    
    // только так работают следующих хуки, без этого трюка они не выполнятся
    include_once GCU_FILES . 'GcuService.php';
    
    // активация плагина
    register_activation_hook(__FILE__, ['GcuService', 'activation']);
    
    // деактивация плагина
    register_deactivation_hook(__FILE__, ['GcuService', 'deactivation']);
    
    // почему-то через uninstall.php не работет, поэтому через хук
    register_uninstall_hook(__FILE__, ['GcuService', 'uninstall']);
} else {
    // … иначе — активировать проверку урлов
    
    // информация о комментарии и комментаторе при добавлении
    add_filter('preprocess_comment', ['GoodCommentsUrl', 'run'], 0);
    
    // после добавления комментария проверим, нужно ли что записать в таблицу «плохих» URL
    add_filter('comment_post', ['GoodCommentsUrl', 'run2'], 0);
}

class GoodCommentsUrl
{
    private static $last_url_info = [ '', '' ];
    
    /**
    * Проверка сайта на качество и обнуление URL в случае проблем
    */
    static function run($a) {
        $current_user = wp_get_current_user();
        
        // для администраторов проверок не делать
        if (isset($current_user->roles) && in_array('administrator', $current_user->roles)) {
            return $a;
        }
        
        // проверим, не комментируется ли администратором
        $user = wp_get_current_user();
        
        // если да, ничего менять не нужно
        if ( in_array('administrator', $user->roles) ) return $a;
        
        // получить настройки плагина
        $gcu_params_str = get_option('gcu_params');
        $gcu_params = $gcu_params_str ? unserialize($gcu_params_str) : [];
        
        // если их нет — установить значения по умолчанию
        
        // начиная с какого значения ИКС разрешать ссылки
        if (!isset($gcu_params['min_iks'])) $gcu_params['min_iks'] = 0;
        
        // если комментатор не задал url…
        if ( empty($a['comment_author_url']) ) {
            // … ничего делать не нужно
            $url = '';
        } else {
            // … иначе — запустить проверки
            
            $url = $a['comment_author_url'];
            
            if ($gcu_params['check_scheme'] && !self::checkScheme($url)) {
                self::$last_url_info = [ $url, 'NotWebURL' ];
                $url = '';
            } else if ( preg_match('!^((https?:|)?//[^/]+).*$!i', $url, $t) ) {
                // получить домен в виде http://site.name или https://site.name
                
                // если схема опущена, принимаем защищённую
                $tURL = (empty($t[1]) ? 'https:' : '') . $t[1];
                
                // проверить, что указанный сайт больше заданного ИКС
                if ($gcu_params['min_iks']) {
                    $iks = self::getIks($tURL);
                    if ($iks < $gcu_params['min_iks']) {
                        if ( empty(self::$last_url_info[0]) ) {
                            self::$last_url_info = [ $url, 'IKS=' . $iks ];
                        } else {
                            self::$last_url_info[1] .= '|' . 'IKS=' . $iks;
                        }
                        $url = '';
                    }
                }
            } else {
                // ...
            }
        }
        $a['comment_author_url'] = $url;
        return $a;
    }
    
    /**
    * Если комментарий был удалён, добавить его в таблицу «плохих» URL
    */
    public static function run2($id, $comment) {
        global $wpdb;
        
        if (self::$last_url_info[0]) {
            $db_tab = [
                'comment_ID' => $id,
                'url' => self::$last_url_info[0],
                'params' => self::$last_url_info[1],
            ];
            
            $wpdb->replace($wpdb->prefix . 'plgn_gcu_url', $db_tab);
            self::$last_url_info = [ '', '' ];
        }
    }
    
    /**
    * Удаление информации о плохом комментарии (если есть)
    */
    public static function deleteСomment($id) {
        global $wpdb;
        $wpdb->delete($wpdb->prefix . 'plgn_gcu_url', ['comment_ID' => $id], ['%d']);
        
        // периодическая оптимизация таблицы
        if (50 == rand(1, 100)) {
            $wpdb->query('OPTIMIZE TABLE ' . $wpdb->prefix . 'plgn_gcu_url');
        }
    }
    
    /**
    * Добавить настройки в основное меню WP
    */
    public static function adminMenu(){
        include_once GCU_FILES . 'GcuAdminManage.php';
        add_options_page('Good Comments URL', 'Настройки GCU', 'manage_options', 'gcu-options', ['GcuAdminManage', 'run']);
    }
    
    /**
    * Добавить настройки к описанию плагина
    */
    public static function gcuLinks($links, $file) {
        $plugin_file = basename(__FILE__);
        $plugin_file_name = basename($file);
        
        if ($plugin_file_name == $plugin_file) {
            $settings_link = '<a href="options-general.php?page=gcu-options">Настройки</a>';
            array_unshift($links, $settings_link);
        }
        return $links;
    }
    
    /**
    * Возвращает статус принадлежности адреса к стандартному веб-URL
    *
    * Адрес ссылки должен начинаться с https://, http:// или просто //.
    */
    public static function checkScheme($url) {
        return preg_match('!^(https?:)?//!', $url);
    }
    
    /**
    * Возвращает статус проверки ИКС
    *
    * 0 (ноль) означает, что данные не получены или не могут быть получены (например, неверный url)
    * FALSE — значение меньше $min_iks
    * TRUE — значение больше или равно $min_iks
    */
    public static function getIks($url) {
        $iks = -2;
        $t = explode('//', $url);
        
        if (!empty($t[1])) {
            include_once GCU_FILES . 'GcuCheckIks.php';
            $iks = GcuCheckIks::getValueFromImage($t[1]);
        }
        
        return is_int($iks) ? $iks : -1;
    }
    
    /**
    * Переводит xml-объект в удобное представление в виде массива
    */
    private static function o2a($object) {
        return @json_decode(json_encode($object), 1);
    }
    
}
