<?php
/**
* Управление настройками плагина
*/

if (!defined('GCU_FILES')) return;

class GcuAdminManage
{
    public static $comments_at_a_time = 50; // по сколько адресов обрабатывать за раз
    
    public static $params = [];
    
    /**
    * 
    */
    public static function run() {
        // настройки плагина
        $gcu_params_str = get_option('gcu_params');
        self::$params = $gcu_params_str ? unserialize($gcu_params_str) : [];
        
        $menu = [
            'sets' => 'Настройки',   // здесь можно установить основные настройки для проверки
            'view' => 'Просмотр «плохих» URL',// анализ таблицы с «плохими» URL
            'ctrl' => 'Управление',  // очистить таблицу плохих URL, перенести данные в таблицу комментарием
        ];
        
        $act = isset($_GET['act']) ? $_GET['act'] : 'sets';
        if (empty($menu[$act])) $act = 'sets';
        
        $arr = [];
        foreach ($menu as $k => $v) {
            if ($act == $k) {
                $arr[] = '<b>' . $v . '</b>';
            } else {
                $arr[] = '<a href="?page=gcu-options&amp;act=' . $k . '">' . $v . '</a>';
            }
        }
        echo '<p><big>Меню:</big> ' . implode(' | ', $arr) . '</p><br>';
        
        $act .= 'f';
        self::$act();
    }
    
    /**
    * Форма для управления настройками
    */
    public static function setsf() {
        // POST-параметры передаются из нашей формы настроек
        if (isset($_POST['gcu_btn'])) {
            $gcu_params = [];
            
            if (!empty($_POST['check_scheme'])) $gcu_params['check_scheme'] = 1;
            
            if (!empty($_POST['min_iks'])) $gcu_params['min_iks'] = (int) $_POST['min_iks'];
            
            // если массив не пуст
            if ($gcu_params) {
                // записать параметры
                $success = update_option('gcu_params', serialize($gcu_params));
            } else {
                // иначе — удалиться из настроек WP
                delete_option('gcu_params');
                $success = true;
            }
        } else {
            // получить содержимое наших опций
            $gcu_params_str = get_option('gcu_params');
            $gcu_params = $gcu_params_str ? unserialize($gcu_params_str) : [];
        }
        
        ?>
        <div class="wrap">
            <h2>Настройки плагина «Good Comments URL»</h2>
            <?php if (isset($success)) {
                if ($success) {
                    echo '<div class="updated"><p>Настройки успешно сохранены.</p></div>';
                } else {
                    echo '<div class="error"><p>Не удалось сохранить настройки.</p><p class="description">Такое сообщение может возникнуть в случае, если Вы сохраняете форму с прежними настройками.</p></div>';
                }
            } ?>
            <form action="" method="post">
                <table class="form-table">
                    <tbody>
                    <tr>
                        <th colspan="2"><label><input type="checkbox" name="check_scheme" value="1"<?= empty($gcu_params['check_scheme']) ? '' : ' checked'?>> Оставлять только веб-адреса</label><br></th>
                        <td><span class="description">Считать «плохим» URL всё, что <b>не</b> начинается с: http://, https://, //<br>
                            Другие параметры при этом проверяться не будут.</span></td>
                    </tr>
                    
                    <tr>
                        <th>Минимальное значение ИКС для разрешённых URL:</th>
                        <td><input type="text" name="min_iks" value="<?= empty($gcu_params['min_iks']) ? 0 : (int) $gcu_params['min_iks'] ?>" maxlength="6" size="12"></td>
                        <td><span class="description">Если комментатор указал сайт, который имеет меньшее значение ИКС, URL будет перемещён в таблицу «плохих» URL.<br>Значение <b>0</b> (ноль) отменяет данную проверку</span></td>
                    </tr>
                    </tbody>
                </table>
                <p class="submit"><input type="submit" value="Сохранить изменения" class="button button-primary" name="gcu_btn"></p>
            </form>
        </div>
        <?php
    }
    
    /**
    * 
    */
    public static function viewf() {
        global $wpdb;
        $cnt = 1000; // по сколько URL выводить на странице
        $page_curr = isset($_GET['paged']) ? (int) $_GET['paged'] : 0;
        
        $q = 'SELECT SQL_CALC_FOUND_ROWS comment_ID, url, params
            FROM ' . $wpdb->prefix . 'plgn_gcu_url
            ORDER BY comment_ID
            LIMIT ' . $page_curr . ', ' . $cnt;
        $tab = $wpdb->get_results($q, ARRAY_A);
        
        $tab_cnt = count($tab);
        $count = $wpdb->get_var('SELECT FOUND_ROWS()');
        
        $page_all = floor($tab_cnt / $cnt);
        
        $tab_str = '';
        
        for ($i = 0; $i < $tab_cnt; $i++) {
            $tab_row = $tab[$i];
            $tab_str .= '<tr>
                <td>' . $tab_row['comment_ID'] . '</td>
                <td>' . htmlspecialchars($tab_row['url']) . '</td>
                <td>' . htmlspecialchars($tab_row['params']) . '</td>
            </tr>';
        }
        
        echo <<<EON
    <div class="wrap">
        <h2>Просмотр «плохих» URL</h2>
EON;
        
        if ($tab_str) {
            echo '
              <table class="widefat striped comments">
                <thead>
                  <tr>
                    <th width="1%">ID комментария</th>
                    <th>URL</th>
                    <th>Параметры</th>
                  </tr>
                </thead>
                
                <tbody>
                  ' . $tab_str . '
                </tbody>
              </table>';
        } else {
            echo '<p>Таблица «плохих» URL пока пуста.</p>';
        }
        
        echo <<<EON
        <div class="tablenav">
            <div class="tablenav-pages">
                <span class="displaying-num">Записей: $count</span> |
                <span class="displaying-num">Страницы:</span>
                <span class="pagination-links">
EON;
        for ($i = 0; $i <= $page_all; $i++) {
            echo '                    <a'
                , $i == $page_curr ? ' class="disabled"' : ''
                , ' href="?page=gcu-options&amp;act=view'
                , $i ? '&amp;paged=' . $i : ''
                , '">' . ($i + 1) . '</a>';
        }
                    
        echo <<<EON
                </span>
            </div>
        </div>
    </div>
EON;
    }
    
    /**
    * 
    */
    public static function ctrlf() {
        global $wpdb;
        
        $cnt = self::$comments_at_a_time;
        
        // с какого значения таблицы комментариев продолжать анализ урлов
        if (isset($_GET['ctrlf_cnt'])) {
            $from = (int) $_GET['ctrlf_cnt'];
        }
        
        if (isset($_POST['gcu_manage_check'])) { // проверка имеющихся комментариев
            $from = 0;
        } else if (isset($_POST['gcu_manage_return'])) { // вернуть записи из таблицы урлов и таблицу комментариев
            // вернём все удалённые ранее урлы в таблицу комментариев
            $q = '
                UPDATE
                    ' . $wpdb->prefix . 'plgn_gcu_url AS tab2,
                    ' . $wpdb->prefix . 'comments AS tab1
                SET tab1.comment_author_url = tab2.url
                WHERE tab1.comment_ID = tab2.comment_ID
            ';
            
            // если запрос выполнен успешно
            if ($wpdb->query($q)) {
                // очистить таблицу
                $q = 'TRUNCATE TABLE ' . $wpdb->prefix . 'plgn_gcu_url';
                $wpdb->query($q);
                $success_text = 'Адреса сайтов из таблицы «плохих» URL успешно перенесены в таблицу комментариев';
            } else {
                $error_text = 'При переносе данных из таблицы URL возникла ошибка: ' . $wpdb->last_error;
            }
        } else if (isset($_POST['gcu_manage_clear'])) { // очистить таблицу урлов
            $q = 'TRUNCATE TABLE ' . $wpdb->prefix . 'plgn_gcu_url';
            
            if ($wpdb->query($q)) {
                $success_text = 'Таблица «плохих» URL очищена';
            } else {
                $error_text = 'Не удалось очистить таблицу урлов';
            }
        }
        
        // проверяем таблицу комментариев на «плохие» URL
        if (isset($from)) {
            $tmp_dir = get_temp_dir();
            
            // Во временный файл будут записываться состояния проверок для доменов.
            // Это позволит избежать лишник запросов на проверки и снизить нагрузку на сервер.
            $tmp_file = $tmp_dir . 'gcu_domains.tmp';
            
            if (file_exists($tmp_file)) {
                // Если со времени последнего изменения файла прошло более
                // часа — удалить файл
                if ((time() - filemtime($tmp_file)) > 60 * 60) {
                    unlink($tmp_file);
                    $host_arr = [];
                } else {
                    $host_arr = unserialize( file_get_contents($tmp_file) );
                }
            } else {
                $host_arr = [];
            }
            
            include_once GCU_FILES . 'GcuCheckIks.php';
            
            /*
                Теоретически, можно значительно ускорить проверку на больших
                таблицах, если добавить:
                … ORDER BY comment_author_url
                Как правило, блоги имеют некое постоянное ядро читателей,
                комментирующих и ссылающихся на свои сайты. Это позволит
                переносить «плохие» URL пачками, либо так же пропускать хорошие.
                С другой стороны, наблюдается интересная тенденция,
                когда в каждом комментарии идёт «предметная» ссылка на сходную
                статью.
                В общем, как задумка — вариант имеет право на существование.
                Возможно, в будущем код будет дополнен.
            */
            $q = 'SELECT SQL_CALC_FOUND_ROWS comment_ID, comment_author_url
                FROM ' . $wpdb->prefix . 'comments
                -- WHERE comment_author_url <> ""
                ORDER BY comment_ID
                LIMIT ' . $from . ', ' . $cnt;
            $tab = $wpdb->get_results($q, ARRAY_A);
            
            $count = $wpdb->get_var('SELECT FOUND_ROWS()');
            
            // Комментарии с «плохими» URL
            $bad_url_arr = [];
            
            foreach ($tab as $v) {
                $status = 0;
                $host_status = '';
                
                if ( ! $v['comment_author_url'] ) {
                    continue;
                }
                
                // проверка наличия схемы: http://, https://, //
                if (!empty(self::$params['check_scheme'])) {
                    if (!GoodCommentsUrl::checkScheme($v['comment_author_url'])) {
                        $status = 1;
                    }
                }
                
                // получение хоста
                $aurl = parse_url($v['comment_author_url'], PHP_URL_HOST);
                
                if (empty($aurl)) {
                    // если неудача — плохой хост
                    $status = 1;
                } else {
                    // перевести хост в нижний регистр
                    $aurl = mb_strtolower($aurl, 'UTF-8');
                }
                
                // использовать ли проверку на минимальный ИКС
                if (!empty(self::$params['min_iks']) && !$status) {
                    if ( isset($host_arr[ $aurl ]) ) {
                        if (preg_match('!\bIKS=\d+\b!', $host_arr[ $aurl ])) {
                            $status += 4;
                        }
                    } else {
                        $iks = GcuCheckIks::getValueFromImage($aurl);
                        if ($iks < self::$params['min_iks']) {
                            $host_status = ($host_status ? $host_status . '|' : '')
                                . 'IKS=' . $iks;
                            $status += 4;
                        } else {
                            if (!isset($host_status)) {
                                $host_status = '';
                            }
                        }
                    }
                }
                
                if (!isset($host_arr[ $aurl ])) {
                    $host_arr[ $aurl ] = $host_status;
                }
                
                // если некорректный URL или слишком малый ИКС —
                // удалить url из таблицы комментариев
                if ($status) {
                    $bad_url_arr[$v['comment_ID']] = [ $v['comment_author_url'], $status, $host_arr[ $aurl ] ];
                }
            }
            
            $from += $cnt;
            if ($from >= $count) {
                $success_text = 'Проверка комментариев успешно завершена. Всего обработано записей: ' . $count;
                unset($from);
            } else {
                $warning_text = 'Пожалуйста, не закрывайте это окно. Выполнено '
                    . $from . ' из ' . $count
                    . '. Проверка комментариев продолжится через пару секунд…';
            }
            
            if ($bad_url_arr) {
                $sql = '';
                
                foreach ($bad_url_arr as $comment_ID => $ars) {
                    list($comment_author_url, $status, $status_str) = $ars;
                    
                    $db_tab = [
                        'comment_ID' => $comment_ID,
                        'url' => $comment_author_url,
                    ];
                    
                    if ($status & 1) {
                        $status_str = 'NotWebURL';
                    }
                    
                    $db_tab['params'] = $status_str;
                    
                    $wpdb->replace($wpdb->prefix . 'plgn_gcu_url', $db_tab);
                }
                
                // удалить запись из таблицы
                $q = 'UPDATE ' . $wpdb->prefix . 'comments
                    SET comment_author_url = ""
                    WHERE comment_ID IN (' . implode(',', array_keys($bad_url_arr)) . ')';
                $wpdb->query($q);
            }
            
            file_put_contents($tmp_file, serialize( $host_arr ));
        }
        
        echo '
<div class="wrap">
    <form action="?page=gcu-options&amp;act=ctrl" method="post">';
        
        if (isset($success_text)) {
            echo '<div class="updated"><p>' . $success_text . '</p></div>';
        } else if (isset($error_text)) {
            echo '<div class="error"><p>' . $error_text . '</p></div>';
        } else if (isset($warning_text)) {
            echo '<div class="update-nag" id="gcu-warning"><p>' . $warning_text . '</p></div>';
        }
        
        echo <<<EON
        <p><input name="gcu_manage_check" type="submit" value="Проверить имеющиеся комментарии" class="button button-primary" onclick="return gcu_submit(this)"><br>
            <span class="description">Проверка будет выполняться порционно, чтобы снизить нагрузку на сервер. После окончания Вы увидите соответствующее уведомление.</span></p>
        
        <p><input name="gcu_manage_return" type="submit" value="Восстановить удалённые URL в таблицу комментариев" class="button button-primary" onclick="return gcu_submit(this)"><br>
            <span class="description">Все «плохие» адреса из таблицы URL будут скопированы обратно в таблицу комментариев. Полезная функция, когда Вы хотите отменить все действия плагина.</span></p>
        
        <p><input name="gcu_manage_clear" type="submit" value="Очистить таблицу URL" class="button button-secondary button-small" onclick="return gcu_submit(this)"><br>
            <span class="description">Очищает таблицу «плохих» URL. Отменить это действие <b>невозможно</b>. Пользуйтесь в случае, когда реально знаете, что хотите получить.</span></p>
    </form>
</div>

EON;
        
        echo <<<EON

<script type="text/javascript">
function gcu_submit(elem) {
    return window.confirm('Вы действительно хотите «' + elem.value + '»?');
}
EON;
        
        // автоматическая перезагрузка страницы, для проверки всех комментариев
        if (isset($from)) {
            echo <<<EON
setTimeout(function() {
    document.getElementById('gcu-warning').innerHTML = '<p>Пожалуйста, подождите…</p>';
    location.href = '?page=gcu-options&act=ctrl&ctrlf_cnt=$from';
}, 3000);
EON;
        }
        
        echo <<<EON
</script>

EON;
    }

}
